#-----------------------------------------------------
# SceneManager
# シーン遷移を管理するモジュール。
#-----------------------------------------------------

module SceneManager
  # インスタンス変数
  @scene = nil   # 現在のシーンオブジェクト
  @stack = []    # シーンのスタック
  @transit = {mode: nil, count: 0} # シーン遷移判定用ハッシュ
  @reserve = {scene: nil, ext: []} # 遷移後のシーン予約
  @outside = [] # 外のマップ
  @tran_reserve = [] #場所移動の予約データ
  @from_map = []
  # 開始
  def self.start
    DataManager.init # データ類の初期化・ロード
    Fonts.init # フォントの初期化
    ScreenShot.init
    VEffect.init
    @scene = first_scene_class.new
  end
  # シーン予約の初期化
  def self.init_reserve
    return @reserve = {scene: nil, ext: []}
  end
  # 最初のシーンクラスを取得
  def self.first_scene_class
    return Scene_Title
  end
  # 現在のシーンの更新
  def self.update
    Key.update
    if Input.key_push?(K_F11)
      $game_window_fullscreen ^= true
      # 全画面表示の設定を保存
      FileUtils.touch("data/bin/system/fullscreen") unless FileTest.exist?("data/bin/system/fullscreen")
      File.open("data/bin/system/fullscreen", "wb") do |file|
        Marshal.dump($game_window_fullscreen, file)
      end
      #----------------------
      Window.full_screen = $game_window_fullscreen rescue nil
    end
    @transit[:count] == 0 ? @scene.update : self.process_transit
    VEffect.update
    Audio.update
    $system.update_timelap
    GC.start
  end
  # 現在のシーンオブジェクトを取得
  def self.scene
    @scene
  end
  # 退避されているシーンオブジェクトを取得
  def self.stacks
    @stack
  end
  # 遷移中の処理
  def self.process_transit
    @transit[:count] -= 1
    if @transit[:count] == 0
      VEffect.fadein(@scene.transition_count)
      Key.unfreeze
      case @transit[:mode]
      when :call
        @stack.push(@scene)
        @scene = @reserve[:scene].new(@reserve[:ext])
      when :goto
        @scene.dispose
        @scene = @reserve[:scene].new(@reserve[:ext])
      when :map_to
        @stack.push(@scene)
        @scene.spriteset.events_dispose unless @scene.spriteset.nil?
        @scene.sprite_dispose
        @scene = @reserve[:scene].new(@reserve[:ext])
      when :return_map
        @scene.dispose
        @scene = @stack.pop
        @scene.load_events_image_stacks
        @scene.create_background
        @scene.trancefer_sprites_co_by_pxy
        @scene.spriteset_load_event
        unless @tran_reserve == []
          @scene.perform_trancepose(@tran_reserve[0], @tran_reserve[1], @tran_reserve[2], @tran_reserve[3], @tran_reserve[4], @tran_reserve[5])
          @tran_reserve = []
        end
        Audio.play_bgm(:map, './data/snd/bgm/生体兵器研究所.wav') if @scene.instance_of?(Scene_Simulation)
      when :return
        @scene.dispose
        @scene = @stack.pop
        unless @tran_reserve == []
          @scene.perform_trancepose(@tran_reserve[0], @tran_reserve[1], @tran_reserve[2], @tran_reserve[3], @tran_reserve[4], @tran_reserve[5])
          @tran_reserve = []
        end
      end
      self.ticktock
      @transit[:mode] = nil
      init_reserve
    end
  end
  # 直接遷移
  def self.goto(scene_class, color = :black, *ext)
    VEffect.fadeout(color, @scene.transition_count)
    Key.freeze
    @reserve[:scene] = scene_class
    @reserve[:ext] = ext
    @transit[:mode]  = :goto
    @transit[:count] = @scene.transition_count
  end
  # シーン呼び出し(コールバック用)
  def self.call(scene_class, color = :black, *ext)
    raise('called scene is overlapping; called twice.') if @transit[:count] != 0
    VEffect.fadeout(color, @scene.transition_count)
    Key.freeze
    @from_map << false
    @reserve[:scene] = scene_class
    @reserve[:ext] = ext
    @transit[:mode]  = :call
    @transit[:count] = @scene.transition_count
  end
  # 呼び出し元へ戻る
  def self.return(color = :black)
    raise('called scene is overlapping; called twice.') if @transit[:count] != 0
    VEffect.fadeout(color, @scene.transition_count)
    Key.freeze
    @transit[:mode] = @from_map.last ? :return_map : :return
    @from_map.pop
    @transit[:count] = @scene.transition_count
  end
  # マップからの呼び出し
  def self.map_to(scene_class, color = :black, *ext)
    VEffect.fadeout(:black, @scene.transition_count)
    Key.freeze
    @from_map << true
    @reserve[:scene] = scene_class
    @transit[:mode]  = :map_to
    @transit[:count] = @scene.transition_count
  end
  # 昼夜レイヤーの表示
  def self.ticktock
    if @scene.instance_of?(Scene_Map)
      VEffect.change_hoe(:night, 153, 0) if $system.time_for_map[@scene.root_id] == :night && self.outside
    else
      VEffect.change_hoe(:default, 0, 0)
    end
  end
  # 外判定
  def self.outside
    @outside.each{|outside|
      return true if $player.map_id == outside
    }
    return false
  end
  # 屋外のマップIDを追加
  def self.set_outside(id)
    @outside << id
    @outside.uniq!
  end
  # 場所移動の予約（マップ移動シーン用）
  def self.trancepose_reserve(map_id, x, y, dir, fade, duration)
    @tran_reserve = [map_id, x, y, dir, fade, duration]
  end
  # 呼び出しスタックのクリア
  def self.clear
    @stack.each{|stack| stack.dispose rescue nil}
    @stack.clear
    @stack = []
  end
  # ゲーム終了
  def self.exit
    DataManager.remove_temp
    @scene = nil
    exit!
  end
end
